<?
  // Constant definition for processing ----------------------------------------
  define( "VIESSMANN_GERAETEKENNUNG",   chr(0x00).chr(0xF8) );

  include( "ViessmannVariables.inc.php");

  // Get XML
  $ViessmannDeviceID = GetValueString( VIESSMANN_VARIABLE_DEVICE );
  $DeviceXMLName = IPS_GetKernelDir()."scripts\\ViessmannDevice".$ViessmannDeviceID.".xml";
  try
  {
    $ViessmannDeviceXML = simplexml_load_file( $DeviceXMLName );
  }
  catch (Exception $e)
  {
    $ViessmannDeviceXML = "";
  }

  function ViessmannGetCommand( $Command )
  {
    $CommandData = array();

    global $ViessmannDeviceXML;
    if ( !isset( $ViessmannDeviceXML ) OR $ViessmannDeviceXML == "" )
      return false;

    $CommandData["deviceid"] = $ViessmannDeviceXML["id"];
    $CommandData["protocol"] = $ViessmannDeviceXML["protocol"];

    // Get Command details
    $ViessmannDeviceCommands = $ViessmannDeviceXML->commands[0];

    foreach($ViessmannDeviceXML->commands[0] as $a => $XMLCommand )
    {
      if ( $XMLCommand["name"] == $Command )
      {
        $CommandData["name"]        = $Command;
        $CommandData["action"]      = $XMLCommand["action"];
        $CommandData["address"]     = $XMLCommand->address[0];
        $CommandData["length"]      = $XMLCommand->length[0];
        $CommandData["description"] = $XMLCommand->description[0];
        $CommandData["format"]      = $XMLCommand->format[0];
        break; // for
      }
    }
    if ( isset( $CommandData["name"] ) )
      return $CommandData;
    else
      return false;
  }

  function ViessmannResultToHexDisplay( $Result )
  {
    $ResultConv = "";
    for( $x=0; $x < strlen($Result); $x++)
    {
      if ( ord($Result[$x]) < 10 ) $ResultConv .= "0";
      $ResultConv .= strtoupper(dechex(ord($Result[$x])));
    }
    return $ResultConv;
  }

  function ViessmannConvertData( $Command, $Data )
  {
	  global $ViessmannDeviceID;
	  $ViessmannCommand = ViessmannGetCommand( $Command, $ViessmannDeviceID );
	  if ( $ViessmannCommand === false ) return false;
	  $Format = $ViessmannCommand["format"];
  
	  switch( $Format )
	  {
 		  case "Boolean":
		    if ( ord( $Data[0] ) == 0 )
		       return false;
			   else
				   return true;
		      break;
		  case "Temperature10":
			  $lowByte = ord( $Data[0] );
			  $highByte = ord( $Data[1] );
			  return (( $highByte * 256 ) + $lowByte ) / 10;
			  break;
		  case "Temperature100":
			  $lowByte = ord( $Data[0] );
			  $highByte = ord( $Data[1] );
			  return (( $highByte * 256 ) + $lowByte ) / 100;
			  break;
		  case "Count" or "Mode":
			  $Count = 0;
			  $HexString = ViessmannResultToHexDisplay( $Data );
			  for ( $x=0; $x < strlen( $Data ); $x=$x+2 )
			  {
				 $hex = substr( $HexString, $x,2 );
				 $dec = hexdec( $hex );
				 switch ( $x )
				 {
					case 0:
					  $Count = $Count + $dec;
					  break;
					case 2:
					  $Count = $Count + ( $dec * 256 );
					  break;
					case 4:
					  $Count = $Count + ( $dec * 65536 );
					  break;
					case 6:
					  $Count = $Count + ( $dec * 16777216 );
					  break;
					default:
					  break;
			    }
			  }
			  return $Count;
			  break;
			case "BooleanPercent":
			  return "BOOLEANPERCENT convertion not coded yet.";
			  break;
			case "Triple":
			  return "TRIPLE convertion not coded yet.";
			  break;
			case "Mode":
			  return hexdec(ord($Data[0]));
			  break;
		   case "ControlID":
		     $Result = "";
           for( $x=0; $x < strlen($Data); $x++)
           {
             if ( ord($Data[$x]) < 10 ) $Result .= "0";
             $Result .= strtoupper(dechex(ord($Data[$x])));
           }
           return $Result;
           break;
  		   default:
			  return "";
			  break;
		}
		return $ConvertedResult;
  }
  
  function ViessmannGetData( $Command )
  {
	 $Run = 1; // Maximum time a recheck is done, if Ping (0x05 is returnvalue)
	 // get Command Data
 	 $ViessmannCommand = ViessmannGetCommand( $Command );
	 if ( $ViessmannCommand === false ) return false;
	 
	 // clear previous result
	 do
	 {
	   SetValueString( VIESSMANN_VARIABLE_RESULT, "" );
	   // create command string and send to COM Port
	   $CommandString = "R:".$ViessmannCommand["protocol"].":".$ViessmannCommand["address"].":".$ViessmannCommand["length"].": ";
		$StartTime = microtime();
		$RunTime = 0;
		SetValueString( VIESSMANN_VARIABLE_COMMAND, $CommandString );
	   // wait for execution
      $status = getvalue( VIESSMANN_VARIABLE_COMMAND );
      while ($status != "" AND $RunTime < 1 )
		{
		  $status = getvalue( VIESSMANN_VARIABLE_COMMAND );
		  $RunTime = microtime() - $StartTime;
		}
      // get result
      $Result = GetValue( VIESSMANN_VARIABLE_RESULT );
      if ( $Result != chr(0x05) ) $Run = -1;
      $Run--;
	 } while ( $Run > 0 );
	 return $Result;
  }
  
  function ViessmannSetData( $Command, $Data )
  {
	 // get Command Data
 	 $ViessmannCommand = ViessmannGetCommand( $Command );
	 if ( $ViessmannCommand === false ) return false;
	 if ( $ViessmannCommand["action"] != "ReadWrite" ) return false;
	 // clear previous result
    SetValueString( VIESSMANN_VARIABLE_RESULT, "" );
    // create command string and send to COM Port
  	 $CommandString = "W:".$ViessmannCommand["protocol"].":".$ViessmannCommand["address"].":".$ViessmannCommand["length"].":".$Data;
    SetValueString( VIESSMANN_VARIABLE_COMMAND, $CommandString );
    // wait for execution
    $status = getvalue( VIESSMANN_VARIABLE_COMMAND );
    while ($status != "") $status = getvalue( VIESSMANN_VARIABLE_COMMAND );
    // get result
    $Result = GetValue( VIESSMANN_VARIABLE_RESULT );
    return $Result;
  }
  
  function ViessmannOpen()
  {
    // Open Com Port
    if ( COMPort_GetOpen( VIESSMANN_VARIABLE_COMPORT ) != true )
    {
	   COMPort_SetOpen( VIESSMANN_VARIABLE_COMPORT, true );
	   IPS_ApplyChanges( VIESSMANN_VARIABLE_COMPORT );
    }
  }
  
  function ViessmannClose()
  {
    // Close Com Port
    if ( COMPort_GetOpen( VIESSMANN_VARIABLE_COMPORT ) != false )
    {
	   COMPort_SetOpen( VIESSMANN_VARIABLE_COMPORT, false );
	   IPS_ApplyChanges( VIESSMANN_VARIABLE_COMPORT );
    }
  }
  
  function ViessmannSetVariableByCommand( $Command, $Var )
  {
    $Value = ViessmannGetData( $Command );
    if ( $Value === false )
	 {
	   SetValueBoolean( VIESSMANN_VARIABLE_CONNECTOK, false );
	 }
    else
    {
      SetValueBoolean( VIESSMANN_VARIABLE_CONNECTOK, true );
	   SetValue( $Var, ViessmannConvertData( $Command, $Value ) );
	 }
  }

?>